@@ -21,8 +21,8 @@ import sys
 import json
 import logging
 
-
-from typing import Dict, Optional, List, NamedTuple
+from pathlib import Path
+from typing import Dict, Optional
 from collections import defaultdict
 from dataclasses import dataclass, field
 
@@ -41,7 +41,7 @@ from transformers import (
 from transformers import is_tf_available
 from transformers.trainer_utils import PredictionOutput
 from mc_transformers.utils_mc import MultipleChoiceDataset, Split, processors
-from mc_transformers.data_classes import InputExample
+from mc_transformers.data_classes import WindowPrediction
 
 
 if is_tf_available():
@@ -136,6 +136,12 @@ class DirArguments:
             "help": "Output directory for predictions"
         }
     )
+    save_logits: bool = field(
+        default=False,
+        metadata={
+            "help": "Whether to store logits along with predictions"
+        }
+    )
 
 
 @dataclass
@@ -162,34 +168,12 @@ class WindowArguments:
             "label correction mechanism on windowed features)"
         }
     )
-
-
-class WindowPrediction(NamedTuple):
-    predictions: np.ndarray
-    window_ids: List[int]
-    labels: List[int]
-    label: Optional[int]
-    example: Optional[InputExample]
-
-
-def process_ids(processor, example_ids, window_args):
-    example_ids = [int(id) for id in example_ids]
-    # decode id with processor
-    if callable(getattr(processor, "_decode_id")):
-        if window_args.enable_windowing:
-            # strip truncation window index
-            # ToDo: = Should join + argmax by windows
-            strip = 2
-        else:
-            strip = 0
-        # returns: context_id, question-answer_id
-        decoded_ids = []
-        for ex_id in example_ids:
-            str_ex_id = str(ex_id)
-            str_ex_id = str_ex_id[:len(str_ex_id) - strip]
-            decoded_ids.append(processor._decode_id(int(str_ex_id)))
-        example_ids = decoded_ids
-    return example_ids
+    windows_dir: Optional[str] = field(
+        default=None,
+        metadata={
+            "help": "Output directory for windowed predictions"
+        }
+    )
 
 
 def vote_windowed_predictions(windowed_predictions):
@@ -203,14 +187,18 @@ def vote_windowed_predictions(windowed_predictions):
     return np.array(predictions)
 
 
-def parse_windowed_predictions(processor, args, results, split):
+def parse_windowed_predictions(args, processor, results, split):
     data_args = args['data_args']
+    window_args = args['window_args']
     if split == Split.dev:
+        file_name = "dev_windowed_predictions.json"
         examples = processor.get_dev_examples(data_args.data_dir)
     elif split == Split.test:
+        file_name = "test_windowed_predictions.json"
         examples = processor.get_test_examples(data_args.data_dir)
     else:
-        raise ValueError('Saving training data is cheating!')
+        file_name = "train_windowed_predictions.json"
+        examples = processor.get_train_examples(data_args.data_dir)
 
     predictions = []
     id_to_example_map = {example.example_id: example for example in examples}
@@ -239,12 +227,20 @@ def parse_windowed_predictions(processor, args, results, split):
         )
         predictions.append(windowed_predictions)
 
+    if window_args.windows_dir is not None:
+        Path(window_args.windows_dir).mkdir(parents=True, exist_ok=True)
+        file_path = os.path.join(window_args.windows_dir, file_name)
+        window_preds_str = json.dumps([win.todict() for win in predictions])
+        with open(file_path, 'w') as fout:
+            fout.write(window_preds_str + '\n')
+
     reduced_predictions = vote_windowed_predictions(predictions)
     example_ids, label_ids = zip(*[
         (win_pred.example.example_id, win_pred.example.label)
         for win_pred in predictions
     ])
     return parse_default_predictions(
+        args=args,
         processor=processor,
         example_ids=example_ids,
         label_ids=label_ids,
@@ -252,22 +248,27 @@ def parse_windowed_predictions(processor, args, results, split):
     )
 
 
-def parse_default_predictions(processor, example_ids, label_ids, predictions):
+def parse_default_predictions(args, processor, example_ids, label_ids, predictions):
     # ToDo := Test predictions should not have true label
     # cast to avoid json serialization issues
     example_ids = [processor._decode_id(int(ex_id)) for ex_id in example_ids]
     label_ids = [int(lab) for lab in label_ids]
     label_id_map = {i: chr(ord('A') + int(label)) for i, label in enumerate(processor.get_labels())}
 
+    pred_logits = predictions.tolist()
     predictions = softmax(predictions, axis=1)
     predictions_dict = defaultdict(list)
 
-    for (ex_id, q_id), true_label, preds in zip(example_ids, label_ids, predictions):
+    for (ex_id, q_id), true_label, preds, logits in zip(example_ids, label_ids, predictions, pred_logits):
         pred_dict = {
             "probs": preds.tolist(),
             "pred_label": label_id_map[np.argmax(preds)],
             "label": label_id_map[true_label],
         }
+
+        if args['dir_args'].save_logits:
+            pred_dict.update(**{"logits": logits})
+
         predictions_dict[ex_id].append(pred_dict)
 
     full_ids = ['-'.join([c_id, qa_id]) for c_id, qa_id in example_ids]
@@ -280,7 +281,12 @@ def parse_default_predictions(processor, example_ids, label_ids, predictions):
 
 def save_metrics(metrics, args, split):
     dir_args = args['dir_args']
-    prefix = "eval" if split == Split.dev else "test"
+    prefix = "eval"
+    if split == Split.test:
+        prefix = "test"
+    elif split == Split.train:
+        prefix = "train"
+
     metrics_dict = {}
     output_metrics_file = os.path.join(
         dir_args.metrics_dir,
@@ -289,6 +295,7 @@ def save_metrics(metrics, args, split):
     for key in ["eval_loss", "eval_acc"]:
         if metrics.get(key) is not None:
             metrics_dict[key] = metrics.get(key)
+
     if len(metrics_dict.keys()) == 0:
         logger.info("Neither loss or accuracy found on result dict!")
     else:
@@ -300,14 +307,22 @@ def save_metrics(metrics, args, split):
 
 def save_predictions(processor, results, args, split):
     dir_args, window_args = args['dir_args'], args['window_args']
-    prefix = "eval" if split == Split.dev else "test"
+    prefix = "eval"
+    if split == Split.test:
+        prefix = "test"
+    elif split == Split.train:
+        prefix = "train"
 
     if window_args.enable_windowing:
         predictions_dict, predictions_list = parse_windowed_predictions(
-            processor, args, results, split,
+            args=args,
+            processor=processor,
+            results=results,
+            split=split,
         )
     else:
         predictions_dict, predictions_list = parse_default_predictions(
+            args=args,
             processor=processor,
             example_ids=results.example_ids,
             label_ids=results.label_ids,
@@ -485,6 +500,7 @@ def main():
     )
 
     # Training
+    results = {}
     if training_args.do_train:
         trainer.train(
             model_path=(
@@ -499,7 +515,14 @@ def main():
         if trainer.is_world_master():
             tokenizer.save_pretrained(training_args.output_dir)
 
-    results = {}
+        logger.info("*** Evaluate (train set)***")
+        result = trainer.predict(train_dataset)
+        if trainer.is_world_master():
+            save_results(
+                processor, result, all_args, split=Split.train
+            )
+            results['train'] = result
+
     if training_args.do_eval:
         logger.info("*** Evaluate ***")
         result = trainer.predict(eval_dataset)
